--[[
    Most of this file from Simply Love (SM5):
    https://github.com/quietly-turning/Simply-Love-SM5/blob/1ebad20ef4399879939b5911ec3d9e933001087c/BGAnimations/ScreenGameplay%20underlay/PerPlayer/MeasureCounter.lua
    
    -impl'd into 5.3 by Rin (ry00001), 2020-
]]

local GetNotefieldX = function( plr ) -- ???????
	local p = tostring(plr)
	local game = GAMESTATE:GetCurrentGame():GetName()

	local IsPlayingDanceSolo = (GAMESTATE:GetCurrentStyle():GetStepsType() == "StepsType_Dance_Solo")
	local NumPlayersEnabled = GAMESTATE:GetNumPlayersEnabled()
	local NumSidesJoined = GAMESTATE:GetNumSidesJoined()
	local IsUsingSoloSingles = PREFSMAN:GetPreference('Center1Player') or IsPlayingDanceSolo or (NumSidesJoined==1 and (game=="techno" or game=="kb7"))

	if IsUsingSoloSingles and NumPlayersEnabled == 1 and NumSidesJoined == 1 then return _screen.cx end
	if GAMESTATE:GetCurrentStyle():GetStyleType() == "StyleType_OnePlayerTwoSides" then return _screen.cx end

	local NumPlayersAndSides = tostring( GAMESTATE:GetCurrentStyle():GetStyleType() )
    return THEME:GetMetric("ScreenGameplay","Player".. p .. NumPlayersAndSides .."X")
end

local player = ...
local enabled = LoadModule("Config.Load.lua")("MeasureCounter",CheckIfUserOrMachineProfile(string.sub(player,-1)-1).."/OutFoxPrefs.ini")
local HideRestCounts = not LoadModule("Config.Load.lua")("MeasureCounterBreaks",CheckIfUserOrMachineProfile(string.sub(player,-1)-1).."/OutFoxPrefs.ini")

local streams, prev_measure, MeasureCounterBMT
local current_count, stream_index, current_stream_length

local InitializeMeasureCounter = function()
	-- Streams[pn].Streams is initially set (and updated in CourseMode)
	-- in ./ScreenGameplay in/MeasureCounterAndModsLevel.lua
    streams = Streams[player].Streams
	current_count = 0
	stream_index = 1
	current_stream_length = 0
	prev_measure = 0
end

local GetTextForMeasure = function(self, current_measure, Measures, stream_index)
	-- Validate indices
	if Measures[stream_index] == nil then return "" end

	local streamStart = Measures[stream_index].streamStart
	local streamEnd = Measures[stream_index].streamEnd
	if current_measure < streamStart then return "" end
	if current_measure > streamEnd then return "" end

	local current_stream_length = streamEnd - streamStart
	local current_count = math.floor(current_measure - streamStart) + 1

    local offtext = ""
	local text = offtext
	if Measures[stream_index].isBreak then
		if HideRestCounts == false then
			-- NOTE: We let the lowest value be 0. This means that e.g.,
			-- for an 8 measure break, we will display the numbers 7 -> 0
			local measures_left = current_stream_length - current_count

			if measures_left >= (current_stream_length-1) or measures_left <= 0 then
				text = offtext
			else
				text = "(" .. measures_left .. ")"
			end
			-- diffuse break counter to be Still Grey, just like Pendulum intended
			self:diffuse(0.5,0.5,0.5,1)
		end
	else
		text = tostring(current_count .. "/" .. current_stream_length)
		self:diffuse(1,1,1,1)
	end
	return text, current_count > current_stream_length, self
end


local Update = function(self, delta)
	if not streams.Measures then return end

    local curr_measure = (math.floor(GAMESTATE:GetSongPosition():GetSongBeatVisible()))/4

	-- if a new measure has occurred
	if curr_measure > prev_measure then

		prev_measure = curr_measure
		local text, is_end = GetTextForMeasure(self, curr_measure, streams.Measures, stream_index)

		-- If we're still within the current section
		if not is_end then
			self:settext(text)
		-- In a new section, we should check if curr_measure overlaps with it
		else
			stream_index = stream_index + 1
            text, is_end = GetTextForMeasure(self, curr_measure, streams.Measures, stream_index)
			self:settext(text)
        end
	end
	return self
end

local af = LoadFont('_plex condensed semibold 24px')..{
	Condition=GAMESTATE:IsPlayerEnabled(player),
    OnCommand=function(self)
		if not enabled then return end
		self:horizalign(right)
        local style = GAMESTATE:GetCurrentStyle()
        local styleType = ToEnumShortString(style:GetStyleType())
        local metricName = string.format("PlayerP%i%sX",string.sub(player,-1),styleType)
		local pos = THEME:GetMetric("ScreenGameplay",metricName)
        self:shadowlength(1):xy(pos+(96*2)-(8), _screen.cy)
        -- please excuse the terrifying maths below
        InitializeMeasureCounter()
		self:queuecommand("Update")
	end,
    UpdateCommand = function(self)
        Update(self,0)
        self:sleep(0.02)
        self:queuecommand('Update')
    end,
    CurrentSongChangedMessageCommand = function(self)
        InitializeMeasureCounter()
    end
}

return af